#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <unistd.h>
#include "cca.h"
#include "stdPorts.h"
#include "ports/Integrator_CCA.h"
#include "Driver_CCA.h"

using
  namespace
  integrators::ccaports;

Driver::Driver ()
{
  frameworkServices = 0;
}

Driver::~Driver ()
{
}

int Driver::go () {
  double value;
  int count = 1000;
  double lowerBound = 0.0, upperBound = 1.0;
  
  // Ports
  classic::gov::cca::Port * port;
  Integrator *integrator;
  
  port = frameworkServices->getPort ("IntegratorPort");
  if (port)
    integrator = dynamic_cast < Integrator * >(port);
  if (integrator == 0) {
    std::cerr << "Connected to something other than an Integrator_CCA port\n";
    return -1;
  }

  
  value = integrator->integrate (lowerBound, upperBound, count);
  std::cout << "integral(" << lowerBound << ", " << upperBound << ", " << count << ") = " << value << std::endl;

  value = integrator->integrate (lowerBound*10, upperBound*10, count*10);
  std::cout << "integral(" << lowerBound*10 << ", " << upperBound*10 << ", " << count*10 << ") = " << value << std::endl;

  sleep(1);

  value = integrator->integrate (lowerBound*10, upperBound*100, count*100);
  std::cout << "integral(" << lowerBound*10 << ", " << upperBound*100 << ", " << count*100 << ") = " << value << std::endl;

  // Release ports
  frameworkServices->releasePort ("IntegratorPort");
  return 0;
}

/**
 * The framework passes a pointer to the Services object
 * via this method, which is called by the framework as soon
 * as the component is instantiated.
 */
void
Driver::setServices (classic::gov::cca::Services * services)
{
  if (services != 0)
    {
      frameworkServices = services;

      // Provide a Go port
      classic::gov::cca::PortInfo * portInfo =
        frameworkServices->createPortInfo ("Go", "gov.cca.GoPort", 0);
      if (portInfo != 0)
        frameworkServices->addProvidesPort (this, portInfo);

      // Use an Integrator port
      portInfo =
        frameworkServices->createPortInfo ("IntegratorPort",
                                           "integrators.ccaimpl.Integrator",
                                           0);
      frameworkServices->registerUsesPort (portInfo);
    }
  else
    {
      // Close down if not closed already
      if (frameworkServices != 0)
        frameworkServices->removeProvidesPort ("Go");
      frameworkServices = 0;
    }
}

