/*****************************************************************************
 **                      TAU Portable Profiling Package                     **
 **                      http://www.cs.uoregon.edu/research/paracomp/tau    **
 *****************************************************************************
 ** 	File: 		pg.cpp                                              **
 ** 	Description: 	TAU's proxy generator. It automates the process of  **
 **                     proxy component creation by parsing the PDB file    **
 **			and generating the necessary component interfaces   **
 ** 	Author: 	Sameer Shende <sameer@cs.uoregon.edu>               **
 *****************************************************************************
 */
#ifdef _OLD_HEADER_
# include <fstream.h>
# include <set.h>
# include <algo.h>
#else
# include <fstream>
  using std::ifstream;
  using std::ofstream;
# include <set>
  using std::set;
# include <algorithm>
  using std::sort;
  using std::unique;
#endif
#include <pdbAll.h>
#include <pdbItem.h>
#include <iostream>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
using namespace std;

/* print debugging messages? select -v commandline option to enable */
bool verbose = false;

/* For selective instrumentation */
extern int processInstrumentationRequests(char *fname);
extern bool instrumentEntity(const string& function_name);

/** 
* Create a global variable accessible through a routine that returs a static
* reference.
* The component name1.
*/
string& TheComponentName1(void)
{
  static string name("");
  return name;
}

/** 
* Create a global variable accessible through a routine that returs a static
* reference.
* The component name2.
*/ 
string& TheComponentName2(void)
{
  static string name("");
  return name;
}

/** 
* Create a global variable accessible through a routine that returs a static
* reference.
* The port name 
*/
string& ThePortName(void)
{
  static string name("");
  return name;
}

/** 
* Create a global variable accessible through a routine that returs a static
* reference.
* The AppNamespace name 
*/
string& TheAppNamespace(void)
{
  static string name("");
  return name;
}

/** 
* Routine prototypes
*/
void GenerateSetServices(ostream& outfile, string& classname);
void GenerateSetServicesMasterMind(ostream& outfile, string& classname);
void GenerateCWrappers(ostream& outfile, string& classname);

void FillMethodBody(ostream& outfile, pdbType::argvec& a, const pdbType* returntype, 
		    string& classname, const char * methodname, bool instrument, 
		    const string& prefixmethod, int timerId)
{ /* fill in all the argument stuff */

  string header("  classic::gov::cca::Port *port; \n\
  // Get proxy port \n\
  TAU_COMP_NAME1 *proxy; \n\
  port = frameworkServices->getPort(\""+
  ThePortName()+"Use\");\n\
  if (port) \n\
    proxy = dynamic_cast < TAU_COMP_NAME1 * > (port);  \n\
  if (proxy == 0) \n\
  { \n\
    cerr <<\"Connected to something other than a(n) "+ThePortName()+
    " port\"<< endl; \n\
  }\n\n");
  outfile <<header;
  
  if (instrument) {
    outfile << "\
  if (tautimers[" << timerId << "] == NULL) { \n\
    // Get a Measurement port \n\
    port = frameworkServices->getPort(\"MeasurementPort\");\n\
    performance::ccaports::Measurement *measurement; \n\
    if (port) \n\
      measurement = dynamic_cast < performance::ccaports::Measurement * >(port); \n\
    if (measurement == 0) { \n\
      std::cerr <<\"Measurement port not connected properly\"<<endl; \n\
    } \n\
    classic::gov::cca::ComponentID *cid = frameworkServices->getComponentID(); \n\
    string componentName = cid->toString(); \n\
    tautimers[" << timerId <<"] = measurement->createTimer(componentName \n\
            + string(\"::\") + string(\"";

    if (!prefixmethod.empty()) {
      if (verbose) cout <<"PREFIX = "<<prefixmethod<<endl;
      outfile <<prefixmethod<<"::";
    }
    outfile << methodname<<"\"), string(\""<< returntype->name()<<" (";
    for (pdbType::argvec::const_iterator argiter = a.begin();
	 argiter != a.end(); argiter++) { 
      if (argiter != a.begin()) 
	outfile << ", ";
      outfile << argiter->type()->name();
    }
    outfile << ")\"));" << endl;
    outfile << "    frameworkServices->releasePort(\"MeasurementPort\");\n";
    outfile << "  }\n\n";
    outfile <<"  tautimers[" << timerId << "]->start(); "<<endl;
  }

  outfile << "  "; /* column spacing */

  string resulttag("");
  /* is it void? */
  if (returntype->name().find(string("void")) == string::npos) { 
    /* put the return type */
    outfile << returntype->name() << " result = ";
    resulttag = string(" result");
  }

  outfile << "proxy->"<<methodname<<"(";
  for (pdbType::argvec::const_iterator args = a.begin();
    args != a.end(); args++) {
    if (args != a.begin()) outfile <<",";
    outfile<<args->name();
  }
  outfile << "); \n\n";


  if (instrument) {
    
    outfile << "  tautimers[" << timerId << "]->stop();\n"
               "  // Releasing ports  \n"
               " \n";
  }
  

  string endcode("\n\
  frameworkServices->releasePort(\""+ThePortName()+"Use\"); \n\
  return"); 
  outfile << endcode;
  outfile << resulttag << "; \n";

}

void FillMethodBodyMasterMind(ostream& outfile, pdbType::argvec& a, const pdbType* returntype, 
			      string& classname, const char * methodname) { 
  /* fill in all the argument stuff */

  string header("  classic::gov::cca::Port *port; \n\
  // Get proxy port \n\
  TAU_COMP_NAME1 *proxy; \n\
  port = frameworkServices->getPort(\""+
		ThePortName()+"Use\");\n\
  if (port) \n\
    proxy = dynamic_cast < TAU_COMP_NAME1 * > (port);  \n\
  if (proxy == 0) \n\
  { \n\
    cerr <<\"Connected to something other than a(n) "+ThePortName()+
		" port\"<< endl; \n\
  } \n\
  // Get a Monitor port \n\
  port = frameworkServices->getPort(\"MonitorPort\");\n\
  monitor::ccaports::Monitor *monitor; \n\
  if (port) \n\
    monitor = dynamic_cast < monitor::ccaports::Monitor * >(port); \n\
  if (monitor == 0) { \n\
    std::cerr <<\"Connected to something other than a Monitor port\"<<endl; \n\
  } \n\
  classic::gov::cca::ComponentID *cid = frameworkServices->getComponentID(); \n\
  string tau_name = cid->toString(); \n\
  tau_name+=\"::");
  
  outfile << header;
  outfile << methodname << "(";

  bool previous = false;
  for (pdbType::argvec::const_iterator argiter = a.begin(); argiter != a.end(); argiter++) {
    // print all the argument types
    // except we change all longs/ints/unsigned/etc (number types) to doubles
    // and ignore the rest

    string type = argiter->type()->name();
    pdbItem::type_t kind = argiter->type()->kind();
    
   
    //    if (kind == pdbItem::TY_BOOL || kind == pdbItem::TY_ENUM || 
    //	kind == pdbItem::TY_INT || kind == pdbItem::TY_FLOAT) {
    if (previous)
      outfile <<", ";
    
    outfile << type;
    previous = true;
    //   }
    
  } /* args */

  outfile << ")\";" << endl;
  //outfile <<"  map<string,"<<returntype->name()<<"> params; \n"; 
  outfile << "  map<string,double> tau_params; \n"; 

  for(pdbType::argvec::const_iterator ait = a.begin(); ait != a.end(); ait++) {

    string type = ait->type()->name();
    pdbItem::type_t kind = ait->type()->kind();
    
   
    if (kind == pdbItem::TY_BOOL || kind == pdbItem::TY_ENUM || 
	kind == pdbItem::TY_INT || kind == pdbItem::TY_FLOAT) {

      outfile << "  tau_params.insert(make_pair(\"" << ait->name() << "\", " 
	      << ait->name() << "));\n";
    }

  }
  
  outfile << "  monitor->startMonitoring(tau_name);\n";


  string resulttag("");
  /* is it void? */
  if (returntype->name().find(string("void")) == string::npos)
  { /* put the return type */
    outfile << "  " << returntype->name() << " result = ";
    resulttag = string(" result");
  }

  outfile << "proxy->" << methodname << "(";

  for (pdbType::argvec::const_iterator args = a.begin(); args != a.end(); args++) {
    if (args != a.begin()) outfile << ",";
    outfile << args->name();
  }

  string endcode("); \n\
  monitor->stopMonitoring(tau_name, tau_params); \n\
  // Releasing ports  \n\
  frameworkServices->releasePort(\"MonitorPort\"); \n\
  frameworkServices->releasePort(\""+ThePortName()+"Use\"); \n\
  return");
  outfile << endcode;

  outfile << resulttag << "; \n";
}


/** 
* Generate the proxy component. 
* PDB& p: IN. The pdb file associated with the source code.
* ostream& outfile: IN. The output file stream.
* string& headerfile: IN. The header file to be included.
* string& classname: IN. The class that represents the port. 
*/
void GenerateProxy(PDB& p, ostream& outfile, string& headerfile,  
		   string& classname, const string& prefixmethod, bool masterMind) {
  
  string portType = string("Measurement_CCA.h");

  if (masterMind)
    portType = string("Monitor_CCA.h");

  pdbClass::methodvec methods;
  TheAppNamespace()="TAU2";
  if (!prefixmethod.empty())
  {
    TheAppNamespace()= prefixmethod; 
  }

  /* instead of TAU2, give it a prefix method name to avoid a nameclash! */
  string header("\
#include <iostream> \n\
#include \"cca.h\" \n\
#include \""+portType+"\" \n\
#include \""+headerfile+"\" \n\
#include <unistd.h> \n\
using namespace std; \n\n\
#define TAU_COMP_NAME1 "+TheComponentName1()+ "\n\
#define TAU_COMP_NAME2 \""+TheComponentName2()+ "\" \n\
namespace TAU1 \n\
 { \n\
  namespace "+TheAppNamespace()+"\n\
  { \n\
    class "+classname+" : public virtual TAU_COMP_NAME1, \n\
      public virtual classic::gov::cca::Component \n\
      { \n\
        public:  \n\
		 ");

  outfile << header << endl;

  int timercount = 0;

  // iterate through the list of classes
  for (PDB::classvec::const_iterator it=p.getClassVec().begin();
       it != p.getClassVec().end(); ++it) {
    
    if ((*it)->fullName().find(TheComponentName1()) != string::npos) {
      methods = (*it)->methods();
      if (verbose) cout << "Its methods are:" <<endl;
      
      // output constructor
      outfile << "\t" << classname << "();\n";

      // output the declarations for each method
      for(pdbClass::methodvec::const_iterator mit = methods.begin(); mit != methods.end(); mit++) {
	     


	if (!(*mit)->func()->isCompilerGenerated() &&
	    (*mit)->func()->access() == pdbItem::AC_PUB && 
	    (*mit)->func()->name().find(string("setServices")) == string::npos) {


  	  const pdbType *t = (*mit)->func()->signature();

          if (verbose) cout << (*mit)->func()->name() << endl;
  	  if (verbose) cout <<"****************"<<endl;
          if (verbose) cout <<"Signature: "<<(*mit)->func()->signature()->name()<<endl;	
  	  if (verbose) cout <<"Return type = "<<t->returnType()->name()<<endl;
	 
	  bool isconstructordtor = false;

	  if ((*mit)->func()->specialKind() == pdbItem::RS_CTOR) {
	    continue; // we shouldn't get here anymore, code below left for historical purposes
	    outfile << "\t" << classname;
	    isconstructordtor = true;
	  }

	  if ((*mit)->func()->specialKind() == pdbItem::RS_DTOR) {
	    outfile << "\t~" << classname;
	    isconstructordtor = true;
	  }

	  if (!isconstructordtor) { /* write the return type iff it is not a dtor/ctor */
	    outfile << "\t" << t->returnType()->name();
	    outfile << "\t" << (*mit)->func()->name();
	  }

	  outfile << "(" ;
  
	  if (t->arguments().empty()) {
	    if (verbose) cout <<"Argument(void)"<<endl;
	    outfile << "void"; 
	  }

  	  for (pdbType::argvec::const_iterator ait=t->arguments().begin();
	       ait!=t->arguments().end(); ++ait) {
	    if (ait != t->arguments().begin()) 
	      outfile << ",";
  	    if (verbose) 
	      cout << "Arguments: " << ait->type()->name() << " name: " << ait->name() << endl;
	    outfile << ait->type()->name() << " " << ait->name();
	  }

	  outfile << ");" << endl;

	  if (!isconstructordtor) { 
	    timercount++;
	  }


	}
      } /* end of public */
      /* add the setServices description here */
      outfile << "\tvirtual void setServices(classic::gov::cca::Services *services); " << endl;
      /* continue with the private data items */

      outfile << "\tprivate: \n\
\t  classic::gov::cca::Services *frameworkServices; \n";


      if (!masterMind) {
	
	outfile << "\t  performance::Timer* tautimers[" << timercount << "];" << endl;
	
	/*	// look at all the other methods declared public
	for(pdbClass::methodvec::const_iterator oit = methods.begin();
	    oit != methods.end(); oit++) {

	  if (!(*oit)->func()->isCompilerGenerated() &&
	      (*oit)->func()->access() == pdbItem::AC_PUB &&
	      (*oit)->func()->name().find(string("setServices")) == string::npos) {
	    
	    
	    if ((*oit)->func()->specialKind() == pdbItem::RS_CTOR)
	      continue;
	    
	    if ((*oit)->func()->specialKind() == pdbItem::RS_DTOR)
	      continue;

	    outfile << "\t  performance::Timer* tautimer_" << (*oit)->func()->name()  <<";" << endl;
	  }
	  
	}
	*/

      }
	


      outfile << "      }; \n";



      outfile << classname << "::" << classname << "() {\n";
      if (!masterMind) {
	outfile << "  for (int i=0; i<" << timercount << "; i++) {\n";
	outfile << "    tautimers[i] = NULL;\n";
	outfile << "  }\n";
      }
      outfile << "}\n\n";

      int timerId = 0;
      
      /* look at all the other methods declared public */
      for(pdbClass::methodvec::const_iterator oit = methods.begin();
                      oit != methods.end(); oit++) {

	//if ((*oit)->func()->isCompilerGenerated()) {
	//  cout << "Compiler Generated: " << (*oit)->func()->name() << endl;
	//}

        if (!(*oit)->func()->isCompilerGenerated() &&
             (*oit)->func()->access() == pdbItem::AC_PUB &&
	     (*oit)->func()->name().find(string("setServices")) == string::npos) {
	  /* start writing the other methods */


	  if ((*oit)->func()->specialKind() == pdbItem::RS_CTOR) {
	    continue; // we already wrote our own
	  }

	  //cout << "Not Compiler Generated: " << (*oit)->func()->name() << endl;
	 
	  bool isctordtor = false;

	  if ((*oit)->func()->specialKind() == pdbItem::RS_CTOR) {
	    outfile << classname << "::" << classname;
	    isctordtor = true;
	  }

	  if ((*oit)->func()->specialKind() == pdbItem::RS_DTOR) {
	    outfile << classname << "::~" << classname;
	    isctordtor = true;
	  }

	  if (!isctordtor) { /* write the return type iff it is not a dtor/ctor */
	    outfile <<(*oit)->func()->signature()->returnType()->name()<< " ";
	    outfile << classname << "::" << (*oit)->func()->name();
	  }
	    
	  outfile << "(";
	  pdbType::argvec a = (*oit)->func()->signature()->arguments();
	  if (a.empty()) { 
	    /* there are no arguments */
	    outfile<<"void";
	  }

	  for (pdbType::argvec::const_iterator argiter = a.begin();
	       argiter != a.end(); argiter++) {
	    if (argiter != a.begin()) 
	      outfile << ",";
	    outfile << " " << argiter->type()->name() << " " << argiter->name() << " ";
	  } /* args */
	  
	  outfile << ")\n{" << endl;
	  /* Body of the method */
	  string funcname((*oit)->func()->name()+ " "+ (*oit)->func()->signature()->name());

	  if (verbose) cout <<"FUNCNAME = "<<funcname<<endl;
	  bool instrument = instrumentEntity((*oit)->func()->name());
	  if (!isctordtor) {
	    /* invoke the routine to fill in the body of this method */
	    if (masterMind)
	      FillMethodBodyMasterMind(outfile, a, (*oit)->func()->signature()->returnType(), classname, (*oit)->func()->name().c_str());
	    else
	      FillMethodBody(outfile, a, (*oit)->func()->signature()->returnType(), classname, (*oit)->func()->name().c_str(), instrument, prefixmethod, timerId++);
	  }
	  
	  outfile << "}" << endl;
	}
      } /* end of methods */
      /* Now write the set services function */
      if (masterMind)
	GenerateSetServicesMasterMind(outfile, classname);
      else
	GenerateSetServices(outfile, classname);

      /* write the end of the name space */
      outfile <<"  } /* namespace "<<TheAppNamespace()<<" */"<<endl;
      outfile <<"} /* namespace TAU1 */\n\n"<<endl;
      outfile <<"/* C wrappers for the component */"<<endl;
  
      /* Finally, write the C wrapper functions */
      GenerateCWrappers(outfile, classname);


    } /* find the component name in the list of classes */    
	
  } /* iterate over all class names */
  

} /* end of GenerateProxy */

/** 
* Print the setServices method of the component.
* ostream& outfile: IN. The output file stream.
* string& classname: IN. The class name of the port.
*/
  void GenerateSetServices(ostream& outfile, string& classname) {
  /* add setServices method to the class */
  string header("void "+classname+"::setServices (classic::gov::cca::Services *services)\n{\n");
  outfile<<header;
  string body("  if (services != 0) \n\
  {  \n\
    frameworkServices = services; \n\
    classic::gov::cca::PortInfo * portInfo =  \n\
      frameworkServices->createPortInfo (\""+ 
    ThePortName()+
    "Provide\", TAU_COMP_NAME2, 0); \n\
    if (portInfo != 0) \n\
      frameworkServices->addProvidesPort(this, portInfo); \n\
\n\
    portInfo = frameworkServices->createPortInfo(\"" +
      ThePortName()+"Use\", TAU_COMP_NAME2, 0); \n\
    frameworkServices->registerUsesPort(portInfo); \n\
    // Use a Measurement Port \n\
    portInfo = frameworkServices->createPortInfo(\"MeasurementPort\",\n\
      \"performance.ccaports.Measurement\", 0); \n\
    frameworkServices->registerUsesPort(portInfo); \n\
  }\n\
  else \n\
  { \n\
    // Close down services if not closed already \n\
    if (frameworkServices != 0) \n\
      frameworkServices->removeProvidesPort(\""+ThePortName()+
      "Provide\"); \n\
    frameworkServices = 0; \n\
  } /* end of if */\n\
} /* end of setServices */\n");

  outfile<<body;

}

void GenerateSetServicesMasterMind(ostream& outfile, string& classname)
{
  /* add setServices method to the class */
  string header("void "+classname+"::setServices (classic::gov::cca::Services *services)\n{\n");
  outfile<<header;
  string body("  if (services != 0) \n\
  {  \n\
    frameworkServices = services; \n\
    classic::gov::cca::PortInfo * portInfo =  \n\
      frameworkServices->createPortInfo (\""+ 
	      ThePortName()+
	      "Provide\", TAU_COMP_NAME2, 0); \n\
    if (portInfo != 0) \n\
      frameworkServices->addProvidesPort(this, portInfo); \n\
\n\
    portInfo = frameworkServices->createPortInfo(\"" +
	      ThePortName()+"Use\", TAU_COMP_NAME2, 0); \n\
    frameworkServices->registerUsesPort(portInfo); \n\
    // Use a monitor Port \n\
    portInfo = frameworkServices->createPortInfo(\"MonitorPort\",\n\
      \"monitor.ccaports.Monitor\", 0); \n\
    frameworkServices->registerUsesPort(portInfo); \n\
  }\n\
  else \n\
  { \n\
    // Close down services if not closed already \n\
    if (frameworkServices != 0) \n\
      frameworkServices->removeProvidesPort(\""+ThePortName()+
	      "Provide\"); \n\
    frameworkServices = 0; \n\
  } /* end of if */\n\
} /* end of setServices */\n");

  outfile<<body;



}

/** 
* Generate the C wrapper routines that are invoked by the framework.
* ostream& outfile: IN. The output file stream.
* string& classname: IN. The class name of the port.
*/
void GenerateCWrappers(ostream& outfile, string& classname)
{ /* write C wrappers */
  string header("extern \"C\" \n"
		"{ \n"
		"  classic::gov::cca::Component *create"+classname+"() {\n"
		"    classic::gov::cca::Component *wrapper; \n"
		"    TAU1::"+TheAppNamespace()+"::"+classname+" * component; \n"
		"    component = new TAU1::"+TheAppNamespace()+"::"+classname+"();  \n"
		"    wrapper = dynamic_cast < classic::gov::cca::Component *> (component); \n"
		"    return wrapper; \n"
		"  } \n\n"
		"  char **getComponentList() {\n"
		"    static char *list[2]; \n\
    list[0] = \"create"+classname+" "+classname+"\"; \n\
    list[1] = 0; \n\
    return list; \n\
  } \n\
}; /* extern C */ \n");
  outfile <<header;
}



/** 
 * The main program.
 */
int main(int argc, char **argv)
{
   /* process arguments */
  int ch;
  bool errflag = false;
  bool masterMind = false;
  /* program parameters */

  /*
  string pdbfilename("ParallelIntegrator_CCA.pdb");
  string compname("integrators::ccaports::Integrator");	// ((topmost-intro . 18131))
  string compname2("integrators.ccaports.Integrator"); // ((topmost-intro . 18131))
  string portname("IntegratorPort"); // ((topmost-intro . 18216))
  string classname("");		// ((topmost-intro . 18300))
  string headerfile("ports/Integrator_CCA.h");
  string filename("Proxy.cc");
  string selectfile("");
  string prefixmethod("");
  */

  string pdbfilename("");
  string compname("");
  string compname2("");
  string portname("");
  string classname("");
  string headerfile("");
  string filename("Proxy.cc");
  string selectfile("");
  string prefixmethod("");


  while ( ( ch = getopt (argc, argv, "c:t:p:d:o:h:f:x:vmn:")) != -1) {
    /* process the parameters */
    switch (ch) 
      {
      case 'c': compname 	= optarg; break; 
      case 't': compname2 	= optarg; break;
      case 'p': portname 	= optarg; break;
      case 'd': pdbfilename 	= optarg; break;	
      case 'o': filename 	= optarg; break;
      case 'h': headerfile 	= optarg; break;
      case 'f': selectfile 	= optarg; break;
      case 'x': prefixmethod 	= optarg; break;
      case '?': errflag 	= true;   break;
      case 'v': verbose 	= true;   break;
      case 'm': masterMind      = true;   break;
      case 'n': classname       = optarg; break; 
      }
  }

  if (compname == "" || compname2 == "" || portname == "" || pdbfilename == "" || headerfile == "")
    errflag = true;


  if (errflag) {
    cout << "Usage: " << argv[0] << " [Arguments] \n";
    cout << "Generate instrumented proxies for CCA components from a pdb file\n\n";
    cout << "Example: " << argv[0] << " -c integrators::ccaports::Integrator\n";
    cout << "   -t integrators.ccaports.Integrator -n IntegratorProxy\n";
    cout << "   -p IntegratorPort -d ParallelIntegrator_CCA.pdb \n";
    cout << "   -o Proxy.cc -h ports/Integrator_CCA.h -f select.dat\n\n";
    cout << "Required Arguments\n";

    cout << "  -c <component name>     Full name of the component\n";
    cout << "                          (e.g. integrators::ccaports::Integrator)\n";
    cout << "  -t <type name>          Type of component\n";
    cout << "                          (e.g. integrators.ccaports.Integrator)\n";
    cout << "  -p <port name>          Name of port to generate proxy for\n";
    cout << "                          (e.g. IntegratorPort)\n";
    cout << "  -d <pdbfile name>       Name of pdb file created from cxxparse (pdtoolkit)\n";
    cout << "                          (e.g. ParallelIntegrator_CCA.pdb)\n";
    cout << "  -h <header file>        Header file for this port\n";
    cout << "                          (e.g. ports/Integrator_CCA.h)\n";
    
    cout << "\nOptional Arguments\n";
    cout << "  -n <proxy name>         Name of the proxy component\n";
    cout << "                          (default: base of component name + Proxy)\n";
    cout << "  -o <output filename>    Name of output file (default: proxy.cc)\n";
    cout << "  -f <selective instrumentation file>\n";
    cout << "                          Pre-generated Selective instrumentation file\n";
    cout << "  -m                      Generate MasterMind component proxy\n";
    cout << "  -v                      Verbose output\n";
    cout << "  -?                      Output help\n\n";
    
    /*
    cout << "-o <output filename>  Name of output file

	 " -c <component name> -t <type name> -p <port name>" << 
	 " -d <pdbfile name> -o <output file name> -h <header file> -x <prefix-tag>" <<
	 " -f <selective instrumentation file> [-m|-v|-?]\n" <<
	 "e.g., \n" << argv[0] <<
	 " -c integrators::ccaports::Integrator -t integrators.ccaports.Integrator\n" <<
	 " -p IntegratorPort -d ParallelIntegrator_CCA.pdb \n" <<
	 " -o Proxy.cc -h ports/Integrator_CCA.h -f select.dat" << endl;
   
    */
    return 0;
  }


  PDB p((char *)pdbfilename.c_str());
  ofstream outfile(filename.c_str());


  outfile << "//This file is generated by TAU's proxygenerator \n";
  outfile << "// The command line used to generate it was: \n// ";
  for (int i=0; argv[i]; i++) {
    outfile << argv[i] << " ";
  }
  outfile << "\n\n";

  TheComponentName1() = compname;
  TheComponentName2() = compname2;
  ThePortName() = portname;
  if (!p) {  
    cout <<"ERROR: PDB file error!" <<endl;
    exit(1);
  }

  /* Selective instrumentation file */
  if (!selectfile.empty())
  { /* it is specified */
    processInstrumentationRequests((char *)selectfile.c_str());
  }

  /* iterate through the list of classes */
  for (PDB::classvec::const_iterator it=p.getClassVec().begin();
		  it != p.getClassVec().end(); ++it) {
    if (verbose) cout <<"checking Class name "<<(*it)->fullName()<<endl;
    if (verbose) cout <<"comparing with "<<compname<<endl;
    
    if ((*it)->fullName().find(compname) != string::npos) {
      if (verbose) cout << "The name of the component is " << (*it)->fullName() << endl;
      
      if (classname == "") 
	classname = (*it)->name()+"Proxy";
      
      pdbClass::methodvec methods = (*it)->methods();
      if (verbose) cout << "Its methods are:" << endl;
      for(pdbClass::methodvec::const_iterator mit = methods.begin(); 
		      mit != methods.end(); mit++) {
	
	if (!(*mit)->func()->isCompilerGenerated()) {
          if (verbose) cout <<(*mit)->func()->name()<<endl;
  	  if (verbose) cout <<"****************"<<endl;
          if (verbose) cout <<"Signature: "<<(*mit)->func()->signature()->name()<<endl;	
  	  const pdbType *t = (*mit)->func()->signature();
  	  if (verbose) cout <<"Return type = "<<t->returnType()->name()<<endl;
  
	  if (t->arguments().empty()) {
	    if (verbose) cout << "Argument(void)" << endl;
	  }
  	  for (pdbType::argvec::const_iterator ait=t->arguments().begin();
  			  ait!=t->arguments().end(); ++ait) {
	    if (verbose) cout <<"Arguments: "<<ait->type()->name()<<" name: "<<ait->name()<<endl;
	  }
	}
      }
    }    
  }
  
  if (verbose) cout <<"Comp name 1 " <<TheComponentName1()<<endl;
  if (verbose) cout <<"Comp name 2 " <<TheComponentName2()<<endl;

  /* Generate the proxy component */
 
  GenerateProxy(p, outfile, headerfile, classname, prefixmethod, masterMind);

  return 0;
}
	 
